//Module Imports
const { resolve: resolvePath } = require("path");
const { existsSync: fexists, unlinkSync: fremove } = require("fs");
//Local Imports
const storage = require("./storage");
const config = require("../config.json");

function load(cairoId) {
  return storage.getUserByCairoId(cairoId);
}

/**
 * Create a user with a cairoId (should use Dunestorm API to login)
 */
function createUser(cairoId) {
  storage.createUser(cairoId);
}
/**
 * Creates file entry given aspects of a file updated
 */
function uploadFile(cairoId, fileData) {
  return storage.uploadFile(cairoId, fileData);
}
/**
 * Deletes files.
 * Requires cairoId to garuntee permission to delete a file
 * Sorts files by user before deleting to speed up reference updates
 */
function deleteFiles(cairoId, targetFiles) {
  return storage.deleteFiles(cairoId, targetFiles).then((deleteData) => {
    var files = deleteData.files;
    var deleteFails = deleteData.failed;
    files.forEach((file) => {
      try {
        fremove(file.path);
      } catch (e) {
        console.error("Error Deleting File", file.name, "\nPath:", file.path);
        deleteFails.push(`${file._id}`);
      }
    });
    return deleteFails.length > 0 && deleteFails;
  });
}
/**
 * Returns a list of filecairoIds that the user owns
 */
function getOwnedFiles(fileList) {
  var files = new Array(fileList.length);
  fileList.forEach(
    (file, i) =>
      (files[i] = new Promise((resolve, reject) =>
        storage.getFile(userId, file).then(resolve).catch(reject)
      ))
  );
  return Promise.all(files);
}
/**
 * TODO: Impliment Advanced Sharing
 * Shares file with various people, and various permissions
 */
function shareFile(cairoId, targetFile) {
  console.log(cairoId, "requesting to share file");
  console.log(targetFile);
}
/**
 * TODO: Impliment Advanced Sharing
 * Returns all files shared with a user
 */
function getSharedFiles(cairoId) {
  return storage.getSharedFileList(cairoId);
}
/**
 * Checks if a the user is the owner and then toggles the list of files to public
 */
function publicfyFiles(cairoId, files) {
  var publicfyFails = [];
  storage.publicfyFiles(cairoId, files);
  return publicfyFails.length > 0 && publicfyFails;
}
module.exports = {
  createUser,
  uploadFile,
  deleteFiles,
  getOwnedFiles,
  publicfyFiles,
  shareFile,
  getSharedFiles,
  load,
};
